import json
import os
from importlib import resources
from random import randint
from typing import List

import pytest
from pkg_resources import resource_filename

from colibri import (
    AcvExploitationOnly,
    InfinitePowerGenerator,
    LimitedGenerator,
    ProjectData,
    ProjectOrchestrator,
)
from colibri.config import data


def test_field_mixin() -> None:
    scheme = LimitedGenerator.to_template()
    lg: LimitedGenerator = LimitedGenerator.from_template(scheme)
    lg.initialize()
    lg.run(1, 1)


def test_generate_simple_house():
    from colibri import (
        OccupantModel,
        SimplifiedWallLosses,
        ThermalSpaceSimplified,
        WeatherModel,
    )
    from colibri.interfaces import Module

    acv_exploitation_only: AcvExploitationOnly = AcvExploitationOnly(name="acv")
    infinite_power_generator: InfinitePowerGenerator = InfinitePowerGenerator(
        name="infinite_power_generator"
    )
    thermal_space_simplified: ThermalSpaceSimplified = ThermalSpaceSimplified(
        name="thermal_space_simplified"
    )
    simplified_wall_losses: SimplifiedWallLosses = SimplifiedWallLosses(
        name="simplified_wall_losses"
    )
    occupants: OccupantModel = OccupantModel(name="occupants")
    weather: WeatherModel = WeatherModel(
        name="weather",
        scenario_exterior_air_temperatures=[
            randint(5, 20) for _ in range(0, 168)
        ],
    )

    module_collection: List[Module] = [
        acv_exploitation_only,
        occupants,
        infinite_power_generator,
        simplified_wall_losses,
        thermal_space_simplified,
        weather,
    ]

    template = {}
    for module in module_collection:
        module_template = module.to_template()
        template = merge_dicts_recursive(template, module_template)

    # run project from template
    project_orchestrator: ProjectOrchestrator = ProjectOrchestrator(
        name="project1", verbose=False
    )
    project_data: ProjectData = ProjectData(name="project_data", data=template)

    # Add models
    project_orchestrator.add_module(module=project_data)
    project_orchestrator.add_module(module=acv_exploitation_only)
    project_orchestrator.add_module(module=infinite_power_generator)
    project_orchestrator.add_module(module=simplified_wall_losses)
    project_orchestrator.add_module(module=thermal_space_simplified)
    project_orchestrator.add_module(module=occupants)
    project_orchestrator.add_module(module=weather)

    project_orchestrator.create_links_automatically()

    for space in project_data.spaces:
        space.occupation = [
            1,
            0,
            2,
            3,
            1,
            1,
            1,
            0,
            2,
            0,
            1,
            2,
            3,
            0,
            2,
            1,
            3,
            3,
            2,
            2,
            2,
            0,
            3,
            3,
            3,
            0,
            0,
            0,
            1,
            0,
            3,
            3,
            3,
            1,
            2,
            3,
            2,
            1,
            0,
            2,
            1,
            0,
            3,
            0,
            0,
            0,
            0,
            0,
            2,
            3,
            0,
            0,
            0,
            1,
            1,
            2,
            0,
            0,
            1,
            1,
            2,
            1,
            2,
            1,
            0,
            3,
            0,
            0,
            0,
            0,
            0,
            2,
            2,
            0,
            2,
            0,
            3,
            0,
            1,
            2,
            2,
            1,
            0,
            2,
            3,
            3,
            2,
            0,
            0,
            3,
            2,
            0,
            0,
            3,
            0,
            1,
            2,
            3,
            2,
            2,
            1,
            1,
            1,
            0,
            2,
            2,
            0,
            2,
            0,
            3,
            1,
            0,
            1,
            0,
            3,
            3,
            3,
            1,
            0,
            2,
            1,
            3,
            1,
            0,
            0,
            1,
            2,
            2,
            0,
            0,
            2,
            0,
            2,
            0,
            0,
            0,
            1,
            0,
            3,
            2,
            0,
            2,
            0,
            0,
            2,
            3,
            1,
            0,
            3,
            2,
            3,
            3,
            2,
            0,
            2,
            0,
            0,
            0,
            2,
            0,
            2,
            3,
            3,
            1,
            2,
            0,
            1,
            2,
            3,
            2,
            2,
            0,
            2,
            0,
            2,
            2,
            0,
            3,
            3,
            0,
            1,
            0,
            0,
            1,
            3,
            0,
            1,
            3,
            0,
            2,
            3,
            2,
            2,
            0,
            1,
            0,
            0,
            0,
            1,
            3,
            2,
            3,
            0,
            3,
            0,
            1,
            0,
            2,
            0,
            3,
            3,
            3,
            0,
            0,
            0,
            1,
            1,
            3,
            3,
            1,
            0,
            2,
            0,
            3,
            3,
            3,
            3,
            0,
            0,
            3,
            0,
            2,
            3,
            0,
            0,
            0,
            2,
            3,
            0,
            1,
            2,
            2,
            0,
            0,
            0,
            0,
            3,
            2,
            3,
            3,
            0,
            0,
            3,
            0,
            0,
            2,
            3,
            0,
            3,
            0,
            0,
            0,
            2,
            0,
            2,
            0,
            2,
            0,
            3,
            3,
            0,
            1,
            2,
            3,
            0,
            0,
            0,
            2,
            1,
            1,
            1,
            1,
            0,
            2,
            0,
            0,
            2,
            3,
            1,
            2,
            0,
            2,
            3,
            1,
            3,
            3,
            3,
            2,
            3,
            1,
            2,
            0,
            1,
            0,
            0,
            3,
            0,
            2,
            0,
            0,
            3,
            0,
            2,
            2,
            2,
            0,
            2,
            0,
            2,
            1,
            3,
            0,
            1,
            0,
            1,
            1,
            0,
            0,
            3,
            0,
            1,
            0,
            2,
            0,
            3,
            0,
            1,
            1,
            1,
            3,
            0,
            1,
            3,
            2,
            1,
            0,
            1,
            0,
            0,
            3,
            0,
            0,
            2,
            1,
            3,
            3,
            2,
            1,
            2,
            2,
            0,
            0,
            1,
            0,
            1,
            1,
            0,
            1,
            2,
            0,
            1,
            2,
            0,
            2,
            1,
            2,
            1,
            3,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            2,
            1,
            3,
            3,
            1,
            2,
            3,
            0,
            2,
            3,
            0,
            3,
            3,
            2,
            0,
            0,
            1,
            0,
            0,
            0,
            0,
            1,
            2,
            0,
            0,
            1,
            0,
            2,
            0,
            2,
            0,
            0,
            0,
            2,
            0,
            2,
            1,
            0,
            0,
            1,
            3,
            1,
            1,
            3,
            0,
            3,
            2,
            0,
            3,
            0,
            0,
            2,
            0,
            3,
            1,
            1,
            1,
            2,
            0,
            3,
            0,
            1,
            2,
            0,
            0,
            0,
            0,
            1,
            0,
            1,
            0,
            2,
            3,
            1,
            1,
            2,
            3,
            3,
            3,
            0,
            1,
            2,
            3,
            0,
            2,
            1,
            1,
            3,
            0,
            0,
            0,
            0,
            0,
            3,
            0,
            3,
            0,
            1,
            0,
            1,
            0,
            0,
            3,
            0,
            0,
            3,
            1,
            0,
            1,
            0,
            0,
            0,
            0,
            1,
            2,
            1,
            0,
            3,
            0,
            1,
            0,
            0,
            2,
            3,
            0,
            2,
            1,
            0,
            3,
            3,
            1,
            3,
            3,
            2,
            0,
            3,
            2,
            0,
            2,
            1,
            1,
            0,
            1,
            2,
            3,
            0,
            3,
            0,
            1,
            0,
            3,
            2,
            3,
            2,
            0,
            3,
            0,
            3,
            0,
            0,
            2,
            3,
            0,
            2,
            0,
            3,
            1,
            0,
            0,
            0,
            0,
            3,
            1,
            2,
            3,
            0,
            2,
            3,
            0,
            2,
            2,
            0,
            2,
            1,
            0,
            0,
            3,
            3,
            2,
            2,
            2,
            0,
            1,
            0,
            2,
            2,
            3,
            0,
            3,
            0,
            0,
            0,
            1,
            3,
            2,
            0,
            1,
            3,
            2,
            1,
            3,
            0,
            1,
            1,
            0,
            0,
            3,
            3,
            0,
            0,
            2,
            0,
            0,
            1,
            3,
            0,
            0,
            0,
            0,
            0,
            0,
            2,
            0,
            1,
            1,
            0,
            0,
            0,
            2,
            2,
            1,
            3,
            0,
            0,
            0,
            0,
            3,
            2,
            1,
            0,
            0,
            3,
            0,
            3,
            0,
            3,
            1,
            3,
            2,
            2,
            3,
            1,
            2,
            1,
            1,
            3,
            0,
            1,
            0,
            2,
            1,
            1,
            1,
            0,
            3,
            0,
            0,
            2,
            1,
            0,
            0,
            0,
            3,
            1,
            0,
            0,
            1,
            0,
            0,
            2,
            1,
            3,
            2,
            0,
            0,
            0,
            2,
            0,
            0,
            1,
            3,
            1,
            3,
            3,
            2,
            2,
            3,
            2,
            3,
            0,
            0,
            3,
            0,
            3,
            1,
            1,
            0,
            1,
            2,
            3,
            1,
            0,
            0,
            0,
            0,
            1,
            0,
            0,
            1,
            3,
            2,
            2,
            1,
            0,
            2,
            2,
            3,
            1,
            0,
            0,
            2,
            1,
            1,
            0,
            0,
            0,
            0,
            1,
            3,
            3,
            0,
            0,
            0,
            0,
            3,
            2,
            3,
            0,
            1,
            0,
            0,
            0,
            3,
            3,
            3,
            0,
            0,
            0,
            1,
            0,
            0,
            0,
            2,
            3,
            1,
            0,
            0,
            2,
            1,
            3,
            2,
            3,
            3,
            0,
            3,
            2,
            0,
            2,
            3,
            0,
            1,
            0,
            3,
            0,
            3,
            2,
            3,
            2,
            0,
            0,
            0,
            0,
            2,
            0,
            1,
            2,
            0,
            3,
            0,
            2,
            1,
            2,
            0,
            1,
            0,
            3,
            1,
            1,
            2,
            2,
            2,
            1,
            1,
            0,
            3,
            2,
            2,
            0,
            0,
            2,
            0,
            3,
            0,
            0,
            3,
            1,
            2,
            1,
            1,
            0,
            1,
            0,
            0,
            2,
            3,
            2,
            1,
            3,
            2,
            0,
            0,
            1,
            0,
            1,
            0,
            0,
            0,
            2,
            3,
            0,
            0,
            0,
            1,
            3,
            0,
            2,
            3,
            0,
            0,
            2,
            0,
            1,
            0,
            3,
            1,
            0,
            2,
            3,
            1,
            2,
            2,
            0,
            0,
            1,
            0,
            0,
            2,
            0,
            0,
            0,
            0,
            0,
            3,
            0,
            0,
            0,
            0,
            1,
            0,
            0,
            2,
            0,
            0,
            1,
            2,
            2,
            3,
            0,
            1,
            3,
            2,
            0,
            0,
            3,
            3,
            0,
            0,
            3,
            1,
            0,
            3,
            0,
            0,
            3,
            1,
            0,
            0,
            3,
            0,
            1,
            1,
            3,
            3,
            0,
            0,
            3,
            2,
            0,
            1,
            1,
            3,
            0,
            2,
            2,
            1,
            0,
            2,
            2,
            2,
            3,
            0,
            1,
            2,
            3,
            1,
            0,
            1,
            1,
            2,
            1,
            1,
            3,
            0,
            2,
            3,
            0,
            1,
            1,
            0,
            0,
            1,
            0,
            0,
            0,
            2,
            1,
            1,
            1,
            2,
            2,
            3,
            0,
            0,
            3,
            3,
            0,
            1,
            0,
            1,
            1,
            0,
            2,
            3,
            0,
            3,
            0,
            2,
            1,
            3,
            1,
            2,
            0,
            0,
            1,
            0,
            1,
            1,
            2,
            0,
            0,
            2,
            1,
            0,
            0,
            1,
            0,
            0,
            3,
            1,
            2,
            2,
            1,
            1,
            3,
            0,
            0,
            0,
            0,
            0,
            0,
            1,
            0,
            3,
            3,
            3,
            3,
            2,
            0,
            1,
            0,
            2,
            3,
            0,
            2,
            0,
            2,
            2,
            0,
            0,
            2,
            2,
            1,
            1,
            1,
            1,
            1,
            3,
            0,
            3,
            0,
            1,
            1,
            3,
            2,
            0,
            3,
            1,
            0,
            1,
            3,
            2,
            0,
            1,
            1,
            0,
            3,
            0,
            1,
            1,
            0,
            2,
            3,
            3,
            3,
            0,
            3,
            0,
            0,
            0,
            3,
            1,
            3,
            2,
            1,
            2,
            1,
            0,
            0,
            3,
            0,
            0,
            1,
            0,
            0,
            3,
            3,
            1,
            2,
            0,
            1,
            2,
            0,
            2,
            0,
            2,
            1,
            0,
            1,
            3,
            1,
            0,
            2,
            3,
            1,
            0,
            2,
            0,
            0,
            1,
            0,
            0,
            0,
            0,
            2,
            2,
            0,
            3,
            0,
            2,
            0,
            2,
            3,
            0,
            3,
            3,
            0,
            3,
            0,
            1,
            0,
            2,
            2,
            1,
            0,
            2,
            2,
            2,
            0,
            3,
            0,
            3,
            1,
            0,
            0,
            3,
            3,
            1,
            1,
            2,
            3,
            0,
            0,
            0,
            3,
            2,
            0,
            0,
            1,
            0,
            0,
            1,
            0,
            0,
            0,
            3,
            1,
            0,
            2,
            0,
            1,
            0,
            2,
            2,
            1,
            0,
            3,
            3,
            0,
            2,
            2,
            2,
            2,
            0,
            0,
            3,
            1,
            3,
            1,
            3,
            0,
            3,
            2,
            0,
            0,
            0,
            2,
            2,
            0,
            0,
            3,
            2,
            2,
            1,
            0,
            3,
            0,
            0,
            2,
            1,
            1,
            2,
            3,
            0,
            0,
            2,
            2,
            3,
            2,
            0,
            0,
            0,
            3,
            0,
            1,
            3,
            3,
            1,
            2,
            0,
            0,
            2,
            2,
            0,
            3,
            1,
            2,
            0,
            0,
            2,
            0,
            0,
            2,
            1,
            1,
            3,
            0,
            0,
            3,
            1,
            0,
            0,
            2,
            0,
            2,
            0,
            1,
            0,
            0,
            0,
            2,
            0,
            0,
            3,
            0,
            2,
            1,
            0,
            3,
            3,
            3,
            0,
            0,
            1,
            3,
            0,
            3,
            0,
            0,
            3,
            3,
            1,
            2,
            2,
            0,
            3,
            1,
            3,
            2,
            0,
            3,
            3,
            1,
            3,
            3,
            1,
            1,
            2,
            1,
            0,
            1,
            0,
            2,
            3,
            0,
            0,
            2,
            3,
            2,
            2,
            3,
            2,
            1,
            0,
            0,
            0,
            2,
            0,
            3,
            0,
            0,
            2,
            0,
            0,
            0,
            0,
            2,
            3,
            1,
            0,
            3,
            0,
            0,
            0,
            1,
            0,
            0,
            3,
            3,
            0,
            0,
            0,
            0,
            1,
            0,
            3,
            3,
            0,
            0,
            3,
            2,
            1,
            0,
            2,
            2,
            3,
            0,
            0,
            0,
            0,
            1,
            0,
            1,
            3,
            0,
            0,
            0,
            1,
            1,
            2,
            0,
            0,
            0,
            0,
            3,
            2,
            0,
            2,
            0,
            0,
            2,
            3,
            0,
            2,
            0,
            3,
            3,
            1,
            3,
            1,
            2,
            0,
            1,
            1,
            0,
            0,
            2,
            3,
            1,
            0,
            2,
            0,
            0,
            3,
            1,
            0,
            0,
            0,
            0,
            3,
            0,
            0,
            0,
            0,
            0,
            1,
            3,
            0,
            2,
            1,
            1,
            0,
            2,
            1,
            0,
            2,
            1,
            0,
            2,
            3,
            1,
            1,
            0,
            2,
            2,
            3,
            0,
            0,
            1,
            3,
            0,
            0,
            2,
            0,
            1,
            0,
            0,
            0,
            1,
            1,
            3,
            1,
            0,
            0,
            1,
            0,
            1,
            1,
            0,
            0,
            1,
            0,
            0,
            1,
            0,
            0,
            0,
            1,
            0,
            0,
            0,
            3,
            0,
            2,
            3,
            0,
            0,
            0,
            0,
            1,
            1,
            3,
            3,
            0,
            1,
            0,
            0,
            3,
            3,
            2,
            3,
            0,
            1,
            1,
            2,
            0,
            3,
            3,
            2,
            1,
            0,
            0,
            0,
            0,
            0,
            0,
            1,
            1,
            0,
            0,
            0,
            3,
            3,
            2,
            2,
            0,
            0,
            0,
            3,
            1,
            0,
            0,
            0,
            3,
            3,
            0,
            1,
            0,
            0,
            2,
            1,
            1,
            0,
            1,
            3,
            3,
            1,
            0,
            2,
            0,
            0,
            2,
            0,
            2,
            0,
            0,
            3,
            3,
            0,
            0,
            3,
            3,
            2,
            0,
            0,
            2,
            0,
            2,
            2,
            0,
            0,
            1,
            2,
            0,
            1,
            0,
            1,
            0,
            0,
            0,
            2,
            3,
            0,
            2,
            3,
            1,
            0,
            1,
            3,
            1,
            0,
            0,
            3,
            3,
            0,
            0,
            3,
            3,
            1,
            3,
            2,
            3,
            0,
            3,
            2,
            1,
            0,
            3,
            2,
            3,
            1,
            1,
            1,
            3,
            1,
            0,
            0,
            2,
            0,
            3,
            0,
            3,
            1,
            3,
            0,
            0,
            3,
            0,
            0,
            1,
            2,
            0,
            0,
            0,
            0,
            0,
            0,
            3,
            3,
            0,
            0,
            0,
            3,
            3,
            2,
            0,
            0,
            0,
            0,
            2,
            0,
            3,
            3,
            0,
            0,
            0,
            3,
            0,
            0,
            2,
            0,
            0,
            1,
            3,
            2,
            0,
            1,
            0,
            0,
            0,
            1,
            3,
            1,
            0,
            0,
            2,
            2,
            0,
            0,
            1,
            1,
            1,
            3,
            3,
            3,
            0,
            1,
            1,
            1,
            0,
            1,
            0,
            2,
            0,
            2,
            3,
            3,
            3,
            0,
            3,
            2,
            1,
            1,
            0,
            0,
            3,
            2,
            0,
            0,
            0,
            1,
            1,
            3,
            0,
            2,
            0,
            1,
            1,
            0,
            0,
            0,
            3,
            3,
            2,
            0,
            3,
            3,
            3,
            2,
            2,
            1,
            1,
            0,
            2,
            1,
            1,
            3,
            0,
            3,
            2,
            2,
            0,
            0,
            0,
            2,
            0,
            0,
            3,
            0,
            3,
            0,
            0,
            2,
            1,
            1,
            1,
            1,
            1,
            1,
            3,
            0,
            0,
            0,
            0,
            0,
            0,
            3,
            1,
            1,
            0,
            0,
            1,
            1,
            3,
            1,
            0,
            2,
            1,
            2,
            2,
            2,
            2,
            1,
            0,
            0,
            0,
            3,
            3,
            3,
            0,
            0,
            2,
            0,
            1,
            1,
            0,
            2,
            3,
            2,
            2,
            1,
            0,
            0,
            3,
            1,
            3,
            2,
            2,
            0,
            0,
            0,
            2,
            0,
            0,
            0,
            2,
            0,
            1,
            3,
            2,
            0,
            1,
            0,
            0,
            1,
            3,
            0,
            3,
            0,
            0,
            3,
            2,
            2,
            2,
            0,
            0,
            1,
            3,
            2,
            3,
            1,
            2,
            3,
            0,
            0,
            0,
            0,
            3,
            2,
            1,
            0,
            2,
            0,
            3,
            0,
            3,
            0,
            1,
            1,
            0,
            0,
            3,
            0,
            0,
            3,
            0,
            0,
            1,
            1,
            3,
            2,
            1,
            1,
            0,
            0,
            2,
            0,
            1,
            2,
            0,
            0,
            1,
            3,
            2,
            1,
            1,
            1,
            0,
            3,
            3,
            1,
            1,
            0,
            0,
            0,
            2,
            0,
            0,
            0,
            0,
            2,
            3,
            2,
            0,
            1,
            0,
            1,
            2,
            0,
            0,
            2,
            1,
            0,
            0,
            0,
            0,
            0,
            0,
            2,
            2,
            3,
            1,
            0,
            2,
            0,
            2,
            1,
            2,
            2,
            0,
            0,
            2,
            0,
            0,
            3,
            0,
            2,
            0,
            0,
            0,
            2,
            3,
            3,
            3,
            0,
            1,
            1,
            1,
            1,
            2,
            0,
            0,
            2,
            2,
            3,
            2,
            3,
            2,
            3,
            0,
            2,
            3,
            2,
            1,
            0,
            2,
            2,
            3,
            0,
            1,
            0,
            0,
            0,
            0,
            0,
            0,
            2,
            0,
            2,
            0,
            2,
            0,
            3,
            0,
            0,
            2,
            2,
            1,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            1,
            3,
            2,
            0,
            0,
            0,
            3,
            3,
            0,
            0,
            0,
            0,
            3,
            0,
            2,
            0,
            1,
            1,
            3,
            3,
            3,
            3,
            1,
            2,
            0,
            1,
            0,
            1,
            1,
            0,
            0,
            3,
            0,
            0,
            3,
            2,
            0,
            2,
            3,
            0,
            3,
            2,
            0,
            0,
            0,
            3,
            1,
            2,
            2,
            0,
            1,
            2,
            0,
            3,
            1,
            1,
            3,
            1,
            1,
            0,
            0,
            0,
            0,
            0,
            2,
            0,
            0,
            3,
            1,
            3,
            0,
            0,
            0,
            2,
            1,
            1,
            2,
            3,
            2,
            3,
            0,
            1,
            1,
            0,
            0,
            1,
            0,
            1,
            0,
            3,
            0,
            3,
            1,
            2,
            3,
            0,
            0,
            0,
            0,
            2,
            2,
            2,
            0,
            3,
            2,
            0,
            0,
            2,
            3,
            0,
            2,
            3,
            0,
            2,
            0,
            3,
            2,
            1,
            3,
            3,
            0,
            1,
            0,
            0,
            3,
            0,
            0,
            3,
            1,
            0,
            2,
            3,
            2,
            2,
            1,
            0,
            2,
            3,
            2,
            0,
            1,
            0,
            0,
            1,
            2,
            0,
            2,
            0,
            0,
            3,
            2,
            3,
            3,
            0,
            2,
            2,
            0,
            0,
            2,
            2,
            2,
            3,
            2,
            1,
            0,
            1,
            2,
            3,
            0,
            1,
            0,
            2,
            1,
            0,
            3,
            1,
            1,
            3,
            3,
            1,
            0,
            0,
            1,
            3,
            3,
            1,
            1,
            0,
            0,
            3,
            0,
            0,
            2,
            2,
            2,
            3,
            3,
            2,
            0,
            2,
            2,
            0,
            2,
            0,
            1,
            0,
            3,
            2,
            3,
            0,
            2,
            0,
            2,
            1,
            1,
            1,
            1,
            0,
            0,
            1,
            3,
            2,
            2,
            0,
            3,
            0,
            2,
            1,
            0,
            0,
            3,
            3,
            3,
            3,
            0,
            0,
            3,
            0,
            3,
            2,
            3,
            1,
            0,
            0,
            0,
            3,
            0,
            1,
            3,
            0,
            0,
            0,
            0,
            1,
            0,
            0,
            2,
            3,
            3,
            2,
            1,
            0,
            0,
            1,
            0,
            1,
            2,
            0,
            0,
            2,
            1,
            2,
            2,
            0,
            3,
            0,
            0,
            2,
            0,
            0,
            1,
            3,
            0,
            2,
            0,
            1,
            3,
            0,
            0,
            1,
            0,
            0,
            0,
            3,
            1,
            2,
            2,
            3,
            0,
            1,
            2,
            1,
            3,
            0,
            1,
            2,
            0,
            2,
            2,
            0,
            2,
            0,
            0,
            3,
            1,
            0,
            3,
            3,
            1,
            0,
            0,
            3,
            2,
            0,
            1,
            3,
            2,
            2,
            2,
            2,
            0,
            3,
            1,
            0,
            0,
            3,
            2,
            0,
            2,
            2,
            3,
            3,
            0,
            1,
            3,
            2,
            3,
            2,
            0,
            0,
            0,
            2,
            0,
            3,
            2,
            2,
            2,
            0,
            2,
            3,
            1,
            0,
            0,
            0,
            2,
            2,
            0,
            0,
            3,
            2,
            1,
            2,
            0,
            0,
            2,
            0,
            1,
            0,
            2,
            1,
            3,
            2,
            0,
            1,
            0,
            3,
            0,
            3,
            2,
            3,
            3,
            2,
            3,
            3,
            0,
            0,
            2,
            3,
            2,
            0,
            1,
            3,
            0,
            0,
            1,
            3,
            3,
            0,
            1,
            1,
            0,
            1,
            3,
            0,
            3,
            0,
            2,
            0,
            2,
            0,
            3,
            3,
            0,
            0,
            2,
            0,
            3,
            0,
            3,
            0,
            3,
            1,
            1,
            1,
            2,
            0,
            0,
            2,
            0,
            2,
            0,
            3,
            2,
            2,
            0,
            1,
            2,
            1,
            1,
            2,
            2,
            2,
            3,
            2,
            0,
            1,
            2,
            2,
            0,
            3,
            1,
            3,
            2,
            0,
            0,
            1,
            0,
            0,
            1,
            0,
            3,
            1,
            3,
            0,
            2,
            0,
            2,
            0,
            1,
            0,
            0,
            1,
            0,
            0,
            3,
            2,
            2,
            3,
            2,
            0,
            2,
            3,
            0,
            3,
            3,
            3,
            1,
            3,
            2,
            0,
            0,
            1,
            0,
            0,
            1,
            0,
            1,
            0,
            0,
            1,
            0,
            1,
            1,
            0,
            0,
            1,
            1,
            1,
            3,
            3,
            2,
            1,
            3,
            0,
            0,
            3,
            0,
            0,
            0,
            0,
            0,
            2,
            1,
            0,
            3,
            1,
            2,
            0,
            3,
            2,
            0,
            3,
            0,
            2,
            0,
            3,
            3,
            0,
            0,
            1,
            0,
            1,
            1,
            0,
            1,
            2,
            2,
            3,
            2,
            2,
            0,
            2,
            1,
            3,
            1,
            2,
            2,
            1,
            3,
            0,
            3,
            0,
            2,
            0,
            0,
            3,
            0,
            2,
            0,
            0,
            1,
            1,
            3,
            0,
            3,
            1,
            1,
            1,
            0,
            3,
            0,
            1,
            0,
            2,
            1,
            0,
            3,
            3,
            0,
            0,
            0,
            0,
            2,
            0,
            0,
            3,
            0,
            2,
            2,
            3,
            0,
            0,
            3,
            0,
            0,
            0,
            0,
            0,
            3,
            1,
            0,
            0,
            0,
            1,
            3,
            3,
            2,
            3,
            0,
            1,
            1,
            1,
            2,
            3,
            1,
            3,
            3,
            1,
            2,
            0,
            1,
            2,
            3,
            3,
            2,
            0,
            2,
            0,
            3,
            2,
            0,
            0,
            3,
            1,
            2,
            0,
            1,
            1,
            1,
            3,
            1,
            2,
            1,
            0,
            3,
            1,
            2,
            0,
            0,
            3,
            1,
            2,
            0,
            3,
            2,
            2,
            0,
            2,
            0,
            3,
            2,
            2,
            1,
            0,
            3,
            1,
            0,
            3,
            3,
            1,
            1,
            3,
            0,
            0,
            1,
            1,
            3,
            3,
            2,
            1,
            2,
            0,
            1,
            0,
            0,
            2,
            0,
            0,
            3,
            2,
            1,
            3,
            1,
            0,
            0,
            0,
            0,
            1,
            0,
            3,
            1,
            0,
            1,
            0,
            0,
            1,
            2,
            2,
            0,
            0,
            1,
            2,
            0,
            3,
            3,
            2,
            2,
            3,
            0,
            0,
            0,
            3,
            1,
            1,
            0,
            1,
            0,
            0,
            1,
            2,
            0,
            1,
            2,
            0,
            3,
            1,
            2,
            0,
            0,
            3,
            0,
            1,
            2,
            1,
            0,
            1,
            0,
            0,
            1,
            2,
            1,
            3,
            0,
            1,
            3,
            2,
            0,
            0,
            0,
            0,
            0,
            3,
            0,
            0,
            0,
            3,
            3,
            0,
            2,
            2,
            2,
            0,
            3,
            2,
            1,
            0,
            2,
            3,
            0,
            0,
            3,
            0,
            1,
            3,
            0,
            0,
            1,
            3,
            1,
            2,
            3,
            1,
            0,
            0,
            0,
            1,
            0,
            2,
            2,
            1,
            2,
            1,
            1,
            0,
            1,
            0,
            3,
            2,
            0,
            1,
            0,
            1,
            2,
            0,
            0,
            0,
            0,
            2,
            0,
            2,
            3,
            0,
            1,
            0,
            2,
            0,
            0,
            2,
            0,
            1,
            1,
            3,
            0,
            3,
            0,
            0,
            0,
            2,
            0,
            0,
            1,
            2,
            0,
            1,
            0,
            1,
            2,
            1,
            0,
            0,
            3,
            2,
            0,
            0,
            1,
            3,
            1,
            0,
            1,
            2,
            3,
            0,
            1,
            2,
            2,
            2,
            0,
            0,
            3,
            3,
            1,
            1,
            0,
            0,
            0,
            0,
            3,
            3,
            1,
            3,
            0,
            3,
            1,
            2,
            3,
            1,
            2,
            2,
            1,
            0,
            2,
            2,
            0,
            0,
            1,
            1,
            3,
            0,
            3,
            0,
            3,
            0,
            1,
            0,
            0,
            0,
            0,
            2,
            3,
            2,
            1,
            0,
            2,
            0,
            0,
            0,
            2,
            0,
            2,
            0,
            0,
            1,
            2,
            1,
            3,
            2,
            2,
            2,
            2,
            0,
            1,
            0,
            0,
            1,
            0,
            1,
            2,
            0,
            0,
            3,
            0,
            0,
            1,
            1,
            0,
            0,
            2,
            3,
            3,
            1,
            1,
            0,
            0,
            3,
            0,
            3,
            2,
            1,
            0,
            2,
            2,
            0,
            0,
            0,
            3,
            2,
            1,
            0,
            2,
            3,
            2,
            2,
            1,
            0,
            0,
            2,
            2,
            1,
            2,
            0,
            0,
            3,
            3,
            3,
            2,
            2,
            1,
            0,
            0,
            1,
            1,
            0,
            3,
            3,
            0,
            2,
            1,
            0,
            2,
            1,
            2,
            3,
            3,
            3,
            0,
            3,
            2,
            1,
            0,
            0,
            0,
            3,
            0,
            0,
            1,
            2,
            0,
            3,
            0,
            0,
            0,
            0,
            3,
            3,
            0,
            2,
            3,
            2,
            0,
            3,
            0,
            2,
            0,
            0,
            0,
            2,
            0,
            3,
            3,
            1,
            1,
            2,
            3,
            3,
            0,
            0,
            2,
            0,
            2,
            1,
            2,
            3,
            1,
            2,
            2,
            1,
            1,
            0,
            1,
            3,
            1,
            3,
            1,
            1,
            0,
            0,
            3,
            3,
            1,
            3,
            0,
            0,
            0,
            1,
            1,
            3,
            0,
            0,
            3,
            1,
            2,
            1,
            0,
            0,
            2,
            3,
            0,
            3,
            0,
            1,
            3,
            1,
            3,
            0,
            0,
            0,
            1,
            0,
            0,
            0,
            2,
            0,
            3,
            3,
            2,
            3,
            1,
            2,
            1,
            0,
            0,
            0,
            3,
            0,
            0,
            2,
            1,
            1,
            2,
            1,
            0,
            0,
            0,
            3,
            1,
            0,
            0,
            1,
            2,
            3,
            0,
            2,
            0,
            0,
            1,
            1,
            0,
            3,
            1,
            0,
            3,
            2,
            3,
            1,
            0,
            3,
            0,
            3,
            0,
            3,
            1,
            3,
            1,
            2,
            2,
            0,
            0,
            1,
            0,
            0,
            2,
            2,
            0,
            2,
            2,
            2,
            0,
            0,
            0,
            3,
            3,
            0,
            3,
            1,
            0,
            0,
            0,
            1,
            1,
            1,
            3,
            1,
            2,
            0,
            0,
            2,
            3,
            0,
            0,
            2,
            0,
            2,
            1,
            1,
            0,
            3,
            2,
            0,
            2,
            0,
            3,
            0,
            1,
            1,
            1,
            1,
            3,
            0,
            3,
            2,
            0,
            1,
            3,
            0,
            3,
            3,
            3,
            3,
            0,
            1,
            1,
            1,
            2,
            3,
            0,
            0,
            0,
            1,
            3,
            0,
            0,
            0,
            1,
            3,
            0,
            2,
            0,
            0,
            0,
            2,
            3,
            3,
            2,
            2,
            2,
            1,
            3,
            0,
            0,
            0,
            1,
            1,
            3,
            3,
            2,
            0,
            1,
            1,
            1,
            1,
            3,
            2,
            0,
            0,
            0,
            3,
            0,
            3,
            0,
            0,
            3,
            1,
            0,
            1,
            3,
            2,
            2,
            3,
            3,
            1,
            1,
            0,
            0,
            0,
            1,
            0,
            2,
            2,
            1,
            3,
            1,
            0,
            0,
            2,
            1,
            0,
            0,
            1,
            2,
            0,
            0,
            0,
            3,
            2,
            3,
            2,
            1,
            0,
            2,
            0,
            0,
            0,
            1,
            0,
            0,
            0,
            3,
            1,
            0,
            3,
            0,
            0,
            1,
            0,
            3,
            2,
            0,
            0,
            0,
            0,
            2,
            0,
            1,
            1,
            0,
            0,
            2,
            0,
            0,
            2,
            3,
            0,
            2,
            0,
            0,
            2,
            2,
            0,
            0,
            1,
            3,
            2,
            0,
            0,
            2,
            2,
            2,
            1,
            1,
            3,
            2,
            2,
            1,
            0,
            0,
            1,
            3,
            0,
            3,
            3,
            1,
            3,
            0,
            3,
            3,
            0,
            0,
            3,
            2,
            1,
            2,
            2,
            2,
            0,
            1,
            2,
            3,
            3,
            1,
            0,
            2,
            0,
            3,
            0,
            3,
            2,
            0,
            2,
            1,
            0,
            2,
            0,
            0,
            0,
            1,
            0,
            0,
            3,
            1,
            0,
            0,
            1,
            1,
            3,
            0,
            3,
            1,
            1,
            3,
            1,
            3,
            2,
            0,
            0,
            0,
            0,
            2,
            2,
            1,
            1,
            2,
            2,
            1,
            0,
            0,
            0,
            3,
            3,
            3,
            0,
            1,
            2,
            2,
            2,
            0,
            3,
            0,
            3,
            0,
            0,
            1,
            1,
            3,
            2,
            3,
            0,
            0,
            1,
            0,
            3,
            1,
            1,
            3,
            3,
            0,
            0,
            2,
            3,
            0,
            1,
            1,
            3,
            1,
            1,
            2,
            0,
            1,
            0,
            0,
            0,
            1,
            1,
            1,
            0,
            0,
            0,
            2,
            3,
            3,
            0,
            0,
            1,
            0,
            3,
            1,
            0,
            1,
            1,
            0,
            0,
            1,
            2,
            3,
            3,
            3,
            1,
            2,
            2,
            0,
            1,
            0,
            0,
            3,
            2,
            1,
            0,
            3,
            0,
            1,
            3,
            2,
            3,
            0,
            0,
            2,
            0,
            0,
            1,
            3,
            0,
            3,
            3,
            2,
            0,
            0,
            3,
            1,
            1,
            1,
            0,
            0,
            3,
            3,
            0,
            1,
            3,
            1,
            2,
            0,
            0,
            2,
            2,
            3,
            0,
            0,
            1,
            3,
            2,
            2,
            1,
            0,
            2,
            2,
            1,
            2,
            3,
            1,
            0,
            3,
            2,
            0,
            3,
            0,
            1,
            1,
            1,
            0,
            0,
            1,
            3,
            2,
            0,
            0,
            0,
            1,
            1,
            0,
            0,
            2,
            2,
            0,
            1,
            1,
            1,
            2,
            3,
            0,
            0,
            0,
            0,
            3,
            0,
            0,
            1,
            0,
            0,
            0,
            0,
            1,
            0,
            0,
            1,
            0,
            2,
            1,
            0,
            1,
            2,
            3,
            0,
            0,
            3,
            0,
            1,
            0,
            2,
            0,
            2,
            0,
            3,
            0,
            0,
            2,
            0,
            3,
            0,
            2,
            3,
            0,
            2,
            3,
            0,
            1,
            2,
            0,
            0,
            0,
            0,
            0,
            0,
            1,
            2,
            2,
            2,
            1,
            0,
            3,
            2,
            0,
            0,
            1,
            1,
            3,
            2,
            1,
            1,
            0,
            3,
            0,
            2,
            0,
            0,
            2,
            0,
            0,
            0,
            2,
            0,
            2,
            1,
            0,
            1,
            0,
            2,
            0,
            0,
            2,
            0,
            3,
            3,
            0,
            0,
            3,
            3,
            0,
            3,
            2,
            0,
            0,
            3,
            0,
            2,
            2,
            0,
            0,
            0,
            2,
            2,
            2,
            1,
            0,
            1,
            2,
            3,
            2,
            0,
            3,
            2,
            3,
            1,
            1,
            0,
            2,
            0,
            0,
            0,
            2,
            1,
            0,
            3,
            2,
            3,
            0,
            2,
            2,
            0,
            1,
            0,
            2,
            3,
            3,
            2,
            2,
            0,
            0,
            1,
            1,
            0,
            3,
            0,
            3,
            0,
            2,
            0,
            3,
            0,
            1,
            0,
            2,
            1,
            2,
            0,
            0,
            3,
            2,
            0,
            0,
            2,
            0,
            3,
            3,
            0,
            2,
            0,
            1,
            0,
            2,
            1,
            2,
            3,
            1,
            3,
            2,
            0,
            3,
            0,
            2,
            0,
            2,
            0,
            3,
            1,
            3,
            0,
            0,
            0,
            0,
            0,
            1,
            0,
            2,
            3,
            3,
            0,
            0,
            0,
            2,
            0,
            0,
            1,
            0,
            2,
            0,
            0,
            2,
            1,
            1,
            0,
            2,
            0,
            0,
            2,
            0,
            2,
            0,
            0,
            0,
            1,
            3,
            1,
            2,
            0,
            0,
            0,
            0,
            3,
            0,
            2,
            0,
            2,
            0,
            0,
            2,
            0,
            3,
            1,
            3,
            1,
            2,
            0,
            0,
            0,
            0,
            3,
            1,
            0,
            1,
            0,
            2,
            0,
            2,
            2,
            0,
            2,
            3,
            0,
            3,
            3,
            0,
            1,
            1,
            3,
            0,
            2,
            1,
            2,
            1,
            3,
            3,
            0,
            2,
            2,
            1,
            0,
            3,
            0,
            3,
            3,
            2,
            0,
            1,
            3,
            2,
            3,
            1,
            3,
            0,
            0,
            0,
            3,
            0,
            0,
            3,
            3,
            0,
            1,
            0,
            3,
            0,
            0,
            0,
            0,
            0,
            0,
            1,
            1,
            3,
            0,
            2,
            0,
            1,
            2,
            2,
            1,
            0,
            2,
            2,
            2,
            3,
            2,
            0,
            0,
            0,
            2,
            2,
            0,
            0,
            2,
            1,
            0,
            0,
            2,
            0,
            2,
            0,
            0,
            2,
            1,
            0,
            1,
            0,
            0,
            0,
            1,
            0,
            2,
            1,
            0,
            2,
            1,
            1,
            3,
            0,
            0,
            3,
            3,
            1,
            0,
            1,
            0,
            3,
            1,
            1,
            0,
            0,
            0,
            0,
            3,
            3,
            3,
            1,
            0,
            3,
            3,
            0,
            3,
            1,
            1,
            0,
            1,
            1,
            2,
            2,
            2,
            0,
            0,
            1,
            0,
            1,
            2,
            1,
            3,
            0,
            2,
            0,
            0,
            0,
            0,
            0,
            3,
            0,
            3,
            3,
            3,
            0,
            1,
            0,
            1,
            3,
            0,
            1,
            0,
            1,
            0,
            2,
            0,
            0,
            1,
            3,
            0,
            0,
            3,
            1,
            0,
            3,
            2,
            1,
            0,
            1,
            0,
            0,
            3,
            3,
            3,
            0,
            2,
            3,
            0,
            0,
            1,
            3,
            2,
            3,
            3,
            1,
            3,
            0,
            3,
            3,
            0,
            0,
            0,
            0,
            3,
            3,
            2,
            2,
            0,
            1,
            2,
            0,
            0,
            2,
            2,
            3,
            0,
            0,
            1,
            3,
            2,
            0,
            0,
            2,
            1,
            0,
            1,
            0,
            1,
            1,
            0,
            3,
            1,
            0,
            0,
            1,
            0,
            1,
            0,
            3,
            3,
            1,
            1,
            0,
            1,
            3,
            2,
            3,
            0,
            0,
            2,
            0,
            0,
            3,
            0,
            0,
            2,
            0,
            1,
            1,
            1,
            0,
            3,
            0,
            0,
            2,
            1,
            1,
            2,
            2,
            1,
            0,
            1,
            1,
            1,
            2,
            3,
            1,
            2,
            1,
            2,
            3,
            0,
            3,
            0,
            0,
            1,
            0,
            0,
            0,
            3,
            0,
            0,
            0,
            2,
            1,
            0,
            2,
            3,
            3,
            0,
            1,
            3,
            2,
            1,
            3,
            3,
            3,
            0,
            0,
            0,
            1,
            0,
            1,
            0,
            0,
            1,
            2,
            3,
            0,
            0,
            1,
            0,
            3,
            1,
            0,
            0,
            3,
            0,
            3,
            0,
            0,
            3,
            0,
            0,
            1,
            3,
            3,
            3,
            1,
            0,
            0,
            0,
            0,
            0,
            1,
            3,
            1,
            1,
            1,
            0,
            3,
            1,
            0,
            0,
            2,
            1,
            2,
            1,
            2,
            0,
            3,
            0,
            3,
            3,
            0,
            1,
            0,
            3,
            0,
            1,
            2,
            1,
            0,
            0,
            0,
            0,
            2,
            0,
            0,
            0,
            0,
            2,
            0,
            3,
            0,
            2,
            2,
            0,
            1,
            3,
            1,
            0,
            0,
            0,
            2,
            1,
            3,
            0,
            1,
            1,
            1,
            3,
            0,
            0,
            3,
            3,
            1,
            2,
            2,
            0,
            3,
            0,
            1,
            0,
            2,
            0,
            1,
            0,
            2,
            0,
            2,
            2,
            1,
            0,
            3,
            0,
            1,
            0,
            0,
            0,
            0,
            3,
            0,
            1,
            3,
            3,
            2,
            3,
            3,
            1,
            0,
            3,
            0,
            2,
            1,
            0,
            3,
            0,
            0,
            2,
            2,
            1,
            0,
            3,
            0,
            1,
            2,
            0,
            1,
            0,
            0,
            1,
            3,
            3,
            1,
            0,
            3,
            1,
            1,
            2,
            0,
            2,
            1,
            0,
            2,
            3,
            3,
            2,
            3,
            3,
            3,
            1,
            1,
            0,
            1,
            1,
            2,
            3,
            0,
            2,
            1,
            0,
            0,
            2,
            0,
            1,
            0,
            0,
            0,
            2,
            3,
            1,
            0,
            1,
            2,
            1,
            0,
            0,
            1,
            2,
            3,
            1,
            0,
            2,
            2,
            0,
            3,
            1,
            2,
            0,
            3,
            0,
            0,
            1,
            0,
            3,
            0,
            0,
            2,
            1,
            2,
            0,
            2,
            0,
            0,
            3,
            2,
            3,
            3,
            0,
            3,
            0,
            0,
            3,
            3,
            2,
            3,
            2,
            2,
            1,
            0,
            3,
            1,
            1,
            1,
            0,
            0,
            3,
            1,
            1,
            0,
            0,
            0,
            2,
            3,
            0,
            2,
            1,
            0,
            2,
            0,
            0,
            0,
            1,
            0,
            3,
            1,
            3,
            3,
            2,
            2,
            3,
            3,
            1,
            2,
            1,
            0,
            0,
            0,
            3,
            2,
            3,
            3,
            3,
            3,
            1,
            0,
            0,
            0,
            3,
            0,
            1,
            0,
            3,
            2,
            0,
            0,
            0,
            1,
            3,
            2,
            3,
            3,
            3,
            3,
            2,
            2,
            2,
            2,
            0,
            0,
            2,
            0,
            2,
            3,
            0,
            2,
            1,
            1,
            1,
            3,
            2,
            2,
            2,
            2,
            0,
            0,
            3,
            3,
            0,
            0,
            3,
            3,
            2,
            0,
            1,
            0,
            1,
            2,
            0,
            2,
            0,
            1,
            1,
            1,
            0,
            3,
            2,
            3,
            1,
            2,
            0,
            3,
            0,
            0,
            1,
            0,
            1,
            0,
            0,
            1,
            3,
            3,
            0,
            1,
            2,
            3,
            1,
            2,
            2,
            0,
            3,
            0,
            1,
            2,
            0,
            2,
            0,
            0,
            0,
            3,
            0,
            1,
            0,
            3,
            3,
            3,
            1,
            0,
            1,
            0,
            2,
            0,
            0,
            2,
            3,
            0,
            0,
            0,
            1,
            1,
            3,
            2,
            1,
            0,
            2,
            3,
            0,
            0,
            0,
            0,
            2,
            0,
            1,
            1,
            2,
            0,
            2,
            0,
            0,
            3,
            3,
            0,
            3,
            3,
            0,
            3,
            1,
            1,
            2,
            2,
            0,
            1,
            3,
            2,
            2,
            0,
            3,
            1,
            2,
            3,
            2,
            0,
            0,
            2,
            2,
            1,
            2,
            1,
            2,
            2,
            0,
            3,
            3,
            3,
            3,
            2,
            0,
            2,
            0,
            2,
            3,
            0,
            3,
            0,
            1,
            0,
            0,
            0,
            2,
            3,
            3,
            3,
            0,
            2,
            0,
            2,
            2,
            0,
            0,
            2,
            0,
            2,
            2,
            3,
            0,
            0,
            0,
            2,
            0,
            2,
            0,
            1,
            0,
            1,
            0,
            2,
            1,
            1,
            2,
            2,
            2,
            3,
            1,
            0,
            1,
            0,
            0,
            2,
            2,
            3,
            0,
            1,
            1,
            3,
            3,
            1,
            0,
            2,
            0,
            2,
            3,
            3,
            3,
            3,
            0,
            1,
            0,
            1,
            0,
            0,
            1,
            1,
            1,
            0,
            1,
            0,
            2,
            1,
            2,
            0,
            3,
            0,
            0,
            1,
            3,
            0,
            0,
            2,
            0,
            0,
            2,
            1,
            0,
            0,
            1,
            0,
            0,
            1,
            0,
            1,
            3,
            2,
            0,
            0,
            0,
            0,
            3,
            3,
            0,
            0,
            2,
            3,
            1,
            3,
            2,
            2,
            2,
            0,
            3,
            2,
            0,
            2,
            0,
            0,
            1,
            2,
            0,
            2,
            1,
            0,
            2,
            1,
            1,
            2,
            0,
            3,
            0,
            2,
            2,
            1,
            2,
            1,
            3,
            2,
            0,
            2,
            2,
            3,
            2,
            3,
            0,
            1,
            2,
            1,
            0,
            0,
            3,
            0,
            1,
            1,
            1,
            2,
            1,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            3,
            1,
            3,
            1,
            3,
            1,
            0,
            3,
            3,
            3,
            0,
            3,
            0,
            0,
            0,
            0,
            1,
            1,
            0,
            1,
            0,
            1,
            3,
            0,
            1,
            0,
            1,
            0,
            3,
            0,
            2,
            0,
            0,
            0,
            3,
            0,
            2,
            3,
            1,
            3,
            0,
            1,
            2,
            0,
            3,
            2,
            0,
            2,
            0,
            1,
            2,
            2,
            2,
            3,
            1,
            1,
            2,
            2,
            0,
            0,
            0,
            3,
            3,
            3,
            0,
            2,
            1,
            0,
            2,
            3,
            0,
            1,
            1,
            2,
            1,
            0,
            0,
            3,
            0,
            3,
            2,
            0,
            2,
            0,
            3,
            0,
            0,
            0,
            2,
            0,
            0,
            2,
            0,
            0,
            3,
            2,
            2,
            1,
            2,
            1,
            2,
            3,
            0,
            2,
            0,
            0,
            1,
            1,
            2,
            0,
            1,
            0,
            2,
            3,
            2,
            0,
            0,
            0,
            3,
            3,
            3,
            0,
            0,
            0,
            1,
            1,
            3,
            3,
            0,
            3,
            3,
            0,
            1,
            0,
            0,
            0,
            0,
            0,
            3,
            2,
            3,
            0,
            0,
            0,
            2,
            3,
            2,
            2,
            1,
            2,
            1,
            3,
            0,
            1,
            0,
            0,
            1,
            0,
            1,
            2,
            1,
            0,
            0,
            2,
            3,
            0,
            0,
            0,
            0,
            0,
            3,
            3,
            0,
            3,
            3,
            2,
            0,
            0,
            1,
            3,
            3,
            2,
            3,
            3,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            3,
            0,
            3,
            3,
            0,
            3,
            0,
            0,
            1,
            0,
            2,
            3,
            0,
            0,
            2,
            1,
            0,
            1,
            1,
            3,
            2,
            0,
            0,
            2,
            3,
            0,
            0,
            3,
            2,
            1,
            2,
            0,
            1,
            3,
            3,
            1,
            0,
            3,
            2,
            2,
            2,
            2,
            3,
            2,
            0,
            0,
            3,
            0,
            2,
            0,
            0,
            0,
            0,
            0,
            1,
            0,
            1,
            0,
            1,
            2,
            0,
            3,
            1,
            3,
            3,
            3,
            0,
            0,
            0,
            0,
            2,
            0,
            3,
            0,
            2,
            0,
            3,
            1,
            0,
            0,
            0,
            2,
            0,
            1,
            0,
            1,
            0,
            0,
            3,
            0,
            0,
            3,
            0,
            2,
            2,
            0,
            0,
            3,
            0,
            2,
            3,
            3,
            3,
            0,
            3,
            3,
            0,
            0,
            0,
            2,
            0,
            0,
            0,
            3,
            1,
            2,
            0,
            1,
            1,
            3,
            2,
            0,
            0,
            0,
            1,
            2,
            2,
            2,
            0,
            1,
            0,
            0,
            1,
            0,
            0,
            2,
            2,
            0,
            2,
            3,
            2,
            0,
            0,
            0,
            1,
            0,
            2,
            3,
            0,
            0,
            0,
            0,
            2,
            3,
            1,
            3,
            2,
            0,
            2,
            0,
            0,
            0,
            1,
            0,
            1,
            3,
            0,
            0,
            2,
            2,
            0,
            1,
            0,
            3,
            0,
            0,
            0,
            0,
            3,
            2,
            0,
            0,
            0,
            1,
            3,
            2,
            0,
            0,
            3,
            1,
            1,
            1,
            0,
            0,
            2,
            3,
            2,
            3,
            1,
            0,
            3,
            2,
            1,
            1,
            0,
            0,
            0,
            0,
            3,
            3,
            0,
            0,
            0,
            1,
            2,
            0,
            0,
            1,
            2,
            1,
            0,
            0,
            2,
            3,
            3,
            3,
            1,
            0,
            2,
            1,
            0,
            3,
            1,
            0,
            0,
            0,
            3,
            2,
            1,
            0,
            0,
            1,
            0,
            2,
            0,
            1,
            0,
            0,
            3,
            0,
            2,
            0,
            0,
            0,
            0,
            0,
            1,
            2,
            0,
            2,
            1,
            2,
            0,
            3,
            3,
            2,
            1,
            0,
            0,
            0,
            0,
            1,
            0,
            3,
            3,
            0,
            2,
            0,
            0,
            3,
            0,
            2,
            2,
            2,
            0,
            0,
            3,
            0,
            1,
            1,
            0,
            3,
            1,
            0,
            3,
            1,
            0,
            3,
            2,
            2,
            1,
            1,
            0,
            0,
            2,
            2,
            3,
            3,
            0,
            0,
            2,
            2,
            2,
            2,
            0,
            0,
            0,
            0,
            2,
            1,
            0,
            0,
            0,
            2,
            2,
            0,
            2,
            2,
            0,
            1,
            0,
            3,
            2,
            0,
            1,
            0,
            0,
            3,
            0,
            1,
            0,
            0,
            0,
            1,
            0,
            0,
            0,
            3,
            0,
            3,
            0,
            2,
            0,
            2,
            3,
            0,
            0,
            1,
            2,
            3,
            3,
            3,
            2,
            3,
            3,
            0,
            2,
            0,
            1,
            0,
            0,
            1,
            1,
            0,
            1,
            0,
            1,
            1,
            1,
            3,
            3,
            0,
            0,
            0,
            3,
            0,
            2,
            0,
            3,
            0,
            3,
            2,
            3,
            0,
            0,
            0,
            0,
            3,
            0,
            3,
            2,
            1,
            2,
            1,
            3,
            0,
            1,
            2,
            0,
            0,
            3,
            2,
            2,
            1,
            0,
            0,
            2,
            0,
            3,
            2,
            1,
            0,
            0,
            2,
            1,
            1,
            0,
            2,
            2,
            1,
            3,
            1,
            1,
            0,
            1,
            3,
            2,
            2,
            1,
            3,
            0,
            0,
            0,
            0,
            0,
            2,
            0,
            1,
            2,
            0,
            1,
            0,
            0,
            2,
            1,
            3,
            2,
            0,
            3,
            0,
            3,
            0,
            1,
            2,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            3,
            3,
            0,
            0,
            3,
            0,
            0,
            0,
            2,
            2,
            1,
            3,
            0,
            1,
            2,
            1,
            0,
            1,
            0,
            2,
            0,
            3,
            3,
            2,
            2,
            3,
            1,
            1,
            0,
            0,
            0,
            2,
            0,
            1,
            2,
            0,
            0,
            2,
            2,
            0,
            0,
            0,
            0,
            1,
            0,
            1,
            1,
            0,
            3,
            0,
            0,
            0,
            0,
            1,
            0,
            3,
            0,
            0,
            1,
            0,
            1,
            3,
            1,
            0,
            3,
            3,
            3,
            0,
            3,
            0,
            0,
            0,
            2,
            3,
            0,
            3,
            2,
            3,
            0,
            0,
            0,
            3,
            3,
            0,
            3,
            1,
            2,
            0,
            0,
            0,
            0,
            1,
            1,
            2,
            0,
            3,
            3,
            2,
            0,
            2,
            1,
            3,
            3,
            1,
            1,
            0,
            1,
            2,
            3,
            0,
            1,
            1,
            1,
            0,
            0,
            0,
            3,
            3,
            2,
            2,
            0,
            3,
            0,
            0,
            2,
            1,
            0,
            1,
            0,
            2,
            2,
            3,
            3,
            2,
            3,
            3,
            1,
            3,
            2,
            3,
            2,
            0,
            2,
            3,
            1,
            0,
            3,
            2,
            0,
            1,
            3,
            0,
            2,
            3,
            0,
            2,
            0,
            3,
            0,
            1,
            2,
            0,
            0,
            0,
            1,
            2,
            0,
            0,
            2,
            0,
            2,
            2,
            0,
            0,
            2,
            0,
            3,
            1,
            2,
            1,
            1,
            3,
            2,
            0,
            1,
            0,
            0,
            3,
            3,
            0,
            0,
            0,
            3,
            3,
            0,
            1,
            2,
            0,
            0,
            0,
            0,
            0,
            3,
            0,
            1,
            2,
            0,
            3,
            3,
            3,
            1,
            0,
            2,
            1,
            2,
            0,
            0,
            3,
            2,
            3,
            1,
            1,
            3,
            2,
            1,
            1,
            0,
            2,
            2,
            0,
            0,
            3,
            1,
            3,
            1,
            1,
            1,
            3,
            1,
            0,
            1,
            0,
            3,
            3,
            1,
            2,
            1,
            0,
            2,
            3,
            2,
            3,
            3,
            2,
            0,
            1,
            2,
            2,
            0,
            2,
            1,
            1,
            0,
            1,
            0,
            1,
            0,
            0,
            1,
            3,
            0,
            2,
            0,
            1,
            0,
            3,
            0,
            2,
            1,
            0,
            1,
            1,
            1,
            0,
            1,
            1,
            0,
            1,
            3,
            1,
            3,
            2,
            0,
            3,
            0,
            1,
            1,
            1,
            2,
            0,
            0,
            0,
            3,
            3,
            0,
            3,
            3,
            3,
            1,
            1,
            0,
            2,
            0,
            3,
            0,
            3,
            3,
            1,
            0,
            0,
            1,
            0,
            1,
            0,
            0,
            2,
            2,
            1,
            0,
            1,
            0,
            2,
            1,
            1,
            0,
            0,
            0,
            1,
            3,
            2,
            1,
            2,
            0,
            2,
            0,
            3,
            0,
            0,
            0,
            0,
            2,
            2,
            3,
            2,
            0,
            0,
            0,
            0,
            0,
            0,
            2,
            3,
            0,
            1,
            3,
            0,
            2,
            2,
            0,
            2,
            0,
            0,
            2,
            2,
            3,
            0,
            2,
            0,
            1,
            3,
            0,
            0,
            0,
            0,
            2,
            0,
            3,
            0,
            0,
            2,
            0,
            2,
            2,
            2,
            0,
            2,
            0,
            3,
            0,
            1,
            0,
            2,
            1,
            3,
            2,
            0,
            1,
            0,
            1,
            3,
            0,
            1,
            0,
            0,
            0,
            2,
            1,
            0,
            3,
            0,
            2,
            2,
            0,
            0,
            0,
            0,
            0,
            3,
            0,
            0,
            3,
            3,
            2,
            3,
            0,
            0,
            0,
            1,
            1,
            1,
            0,
            2,
            0,
            0,
            0,
            3,
            3,
            1,
            2,
            0,
            1,
            0,
            1,
            0,
            3,
            1,
            3,
            2,
            0,
            3,
            0,
            1,
            2,
            2,
            2,
            0,
            0,
            0,
            0,
            0,
            2,
            0,
            1,
            0,
            3,
            0,
            3,
            3,
            0,
            1,
            3,
            0,
            0,
            0,
            1,
            0,
            3,
            0,
            3,
            0,
            0,
            2,
            1,
            0,
            2,
            3,
            0,
            0,
            3,
            1,
            1,
            3,
            0,
            3,
            0,
            2,
            3,
            3,
            1,
            0,
            3,
            1,
            1,
            0,
            3,
            3,
            2,
            0,
            3,
            0,
            1,
            0,
            3,
            3,
            1,
            2,
            0,
            0,
            2,
            3,
            0,
            1,
            1,
            0,
            2,
            3,
            3,
            3,
            1,
            2,
            1,
            1,
            3,
            2,
            3,
            2,
            1,
            0,
            1,
            1,
            2,
            2,
            2,
            0,
            0,
            2,
            2,
            1,
            0,
            2,
            0,
            2,
            3,
            3,
            2,
            2,
            0,
            3,
            1,
            0,
            3,
            0,
            0,
            0,
            0,
            1,
            2,
            1,
            0,
            0,
            2,
            3,
            0,
            1,
            0,
            3,
            3,
            1,
            1,
            1,
            2,
            0,
            3,
            1,
            2,
            2,
            1,
            0,
            3,
            1,
            3,
            0,
            0,
            1,
            1,
            2,
            1,
            1,
            1,
            1,
            3,
            1,
            0,
            0,
            0,
            3,
            3,
            3,
            3,
            0,
            0,
            1,
            3,
            0,
            3,
            2,
            0,
            1,
            1,
            3,
            1,
            2,
            1,
            2,
            0,
            0,
            1,
            2,
            0,
            0,
            3,
            0,
            1,
            0,
            3,
            2,
            1,
            3,
            0,
            1,
            2,
            0,
            2,
            3,
            2,
            0,
            3,
            0,
            3,
            0,
            3,
            3,
            2,
            3,
            0,
            1,
            3,
            1,
            2,
            1,
            3,
            3,
            3,
            2,
            0,
            3,
            0,
            3,
            1,
            1,
            0,
            1,
            0,
            2,
            2,
            2,
            3,
            2,
            0,
            0,
            3,
            1,
            0,
            1,
            0,
            2,
            1,
            0,
            1,
            0,
            2,
            2,
            1,
            0,
            0,
            0,
            2,
            0,
            0,
            0,
            2,
            2,
            0,
            1,
            2,
            2,
            0,
            0,
            0,
            0,
            0,
            0,
            3,
            0,
            2,
            1,
            2,
            0,
            3,
            1,
            0,
            0,
            0,
            0,
            2,
            0,
            0,
            3,
            2,
            1,
            0,
            2,
            0,
            2,
            1,
            2,
            3,
            1,
            3,
            3,
            0,
            2,
            3,
            0,
            0,
            0,
            0,
            0,
            2,
            3,
            1,
            1,
            0,
            2,
            3,
            2,
            3,
            0,
            2,
            2,
            3,
            0,
            1,
            0,
            2,
            0,
            1,
            1,
            0,
            0,
            2,
            0,
            3,
            0,
            1,
            0,
            2,
            0,
            1,
            2,
            0,
            1,
            1,
            0,
            0,
            1,
            0,
            0,
            1,
            0,
            2,
            0,
            0,
            1,
            1,
            3,
            0,
            0,
            1,
            0,
            0,
            2,
            1,
            1,
            0,
            0,
            1,
            2,
            1,
            1,
            0,
            0,
            0,
            0,
            3,
            2,
            0,
            2,
            2,
            2,
            3,
            3,
            3,
            0,
            3,
            0,
            0,
            1,
            2,
            1,
            0,
            3,
            0,
            0,
            2,
            0,
            0,
            0,
            0,
            1,
            0,
            1,
            0,
            0,
            0,
            1,
            2,
            3,
            0,
            3,
            1,
            2,
            2,
            0,
            3,
            2,
            3,
            3,
            1,
            0,
            0,
            2,
            1,
            2,
            0,
            2,
            2,
            0,
            3,
            1,
            2,
            2,
            0,
            1,
            1,
            0,
            3,
            3,
            0,
            0,
            0,
            1,
            0,
            1,
            2,
            1,
            0,
            3,
            3,
            0,
            1,
            1,
            0,
            2,
            0,
            0,
            1,
            0,
            0,
            0,
            0,
            0,
            0,
            3,
            1,
            1,
            0,
            3,
            0,
            3,
            3,
            2,
            0,
            1,
            3,
            0,
            0,
            2,
            0,
            2,
            1,
            3,
            0,
            1,
            0,
            0,
            1,
            0,
            0,
            3,
            2,
            0,
            3,
            3,
            3,
            0,
            2,
            3,
            3,
            2,
            1,
            0,
            0,
            0,
            0,
            0,
            3,
            3,
            2,
            2,
            0,
            2,
            2,
            0,
            2,
            3,
            1,
            0,
            3,
            2,
            2,
            1,
            0,
            1,
            0,
            1,
            0,
            1,
            1,
            3,
            2,
            3,
            3,
            2,
            0,
            3,
            2,
            0,
            2,
            0,
            0,
            1,
            3,
            0,
            2,
            2,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            1,
            0,
            2,
            2,
            2,
            0,
            3,
            0,
            3,
            1,
            0,
            1,
            0,
            2,
            0,
            2,
            0,
            3,
            3,
            0,
            0,
            3,
            0,
            0,
            0,
            0,
            0,
            2,
            2,
            0,
            3,
            0,
            0,
            0,
            2,
            3,
            0,
            2,
            3,
            0,
            2,
            0,
            1,
            0,
            0,
            3,
            3,
            0,
            3,
            2,
            3,
            3,
            1,
            2,
            3,
            2,
            3,
            0,
            3,
            0,
            0,
            3,
            3,
            0,
            1,
            2,
            0,
            3,
            3,
            2,
            0,
            1,
            2,
            2,
            1,
            0,
            0,
            1,
            1,
            1,
            0,
            3,
            0,
            1,
            3,
            1,
            3,
            2,
            2,
            3,
            2,
            1,
            0,
            1,
            1,
            0,
            3,
            3,
            0,
            1,
            2,
            0,
            0,
            0,
            1,
            0,
            0,
            0,
            2,
            0,
            3,
            3,
            2,
            1,
            0,
            3,
            0,
            1,
            0,
            1,
            2,
            0,
            0,
            2,
            3,
            0,
            0,
            1,
            3,
            3,
            3,
            2,
            0,
            3,
            0,
            2,
            0,
            0,
            0,
            0,
            0,
            1,
            2,
            1,
            1,
            0,
            2,
            3,
            2,
            2,
            2,
            0,
            1,
            0,
            2,
            1,
            1,
            1,
            2,
            3,
            2,
            3,
            1,
            3,
            2,
            3,
            2,
            1,
            2,
            0,
            3,
            3,
            3,
            0,
            1,
            0,
            1,
            0,
            0,
            0,
            3,
            2,
            0,
            0,
            3,
            0,
            3,
            0,
            2,
            0,
            2,
            0,
            0,
            1,
            2,
            0,
            2,
            1,
            3,
            2,
            0,
            0,
            0,
            0,
            0,
            2,
            0,
            0,
            3,
            0,
            2,
            3,
            3,
            0,
            1,
            3,
            3,
            3,
            3,
            2,
            1,
            0,
            0,
            0,
            2,
            1,
            3,
            0,
            0,
            3,
            2,
            0,
            1,
            0,
            0,
            1,
            0,
            0,
            0,
            0,
            0,
            1,
            0,
            2,
            1,
            3,
            0,
            1,
            0,
            2,
            0,
            0,
            0,
            1,
            0,
            1,
            0,
            0,
            3,
            1,
            0,
            0,
            2,
            0,
            1,
            1,
            0,
            1,
            1,
            3,
            0,
            0,
            0,
            0,
            3,
            1,
            0,
            1,
            1,
            0,
            1,
            1,
            1,
            2,
            0,
            1,
            0,
            0,
            3,
            2,
            1,
            0,
            0,
            1,
            0,
            1,
            2,
            1,
            0,
            2,
            2,
            0,
            1,
            0,
            0,
            1,
            2,
            1,
            3,
            1,
            3,
            1,
            0,
            1,
            2,
            3,
            3,
            0,
            2,
            0,
            1,
            1,
            3,
            0,
            3,
            3,
            2,
            0,
            1,
            0,
            1,
            0,
            0,
            0,
            1,
            0,
            0,
            3,
            3,
            1,
            2,
            0,
            3,
            3,
            0,
            2,
            0,
            1,
            3,
            0,
            1,
            1,
            0,
            2,
            3,
            0,
            2,
            0,
            2,
            1,
            3,
            2,
            0,
            0,
            2,
            3,
            1,
            3,
            3,
            1,
            2,
            0,
            2,
            0,
            1,
            3,
            0,
            0,
            0,
            0,
            2,
            0,
            0,
            1,
            0,
            3,
            1,
            0,
            1,
            0,
            1,
            0,
            2,
            2,
            1,
            1,
            0,
            1,
            0,
            0,
            2,
            0,
            2,
            2,
            2,
            2,
            0,
            1,
            0,
            0,
            0,
            0,
            2,
            0,
            3,
            1,
            2,
            2,
            3,
            2,
            2,
            0,
            1,
            0,
            2,
            0,
            2,
            0,
            2,
            0,
            0,
            0,
            1,
            2,
            2,
            3,
            3,
            0,
            0,
            2,
            3,
            0,
            3,
            0,
            0,
            0,
            3,
            0,
            3,
            3,
            1,
            0,
            0,
            1,
            2,
            3,
            0,
            0,
            1,
            2,
            2,
            1,
            0,
            2,
            3,
            3,
            1,
            1,
            0,
            1,
            3,
            0,
            3,
            3,
            2,
            2,
            0,
            0,
            0,
            1,
            0,
            1,
            3,
            3,
            0,
            1,
            1,
            1,
            2,
            2,
            1,
            0,
            1,
            2,
            2,
            0,
            2,
            1,
            1,
            0,
            2,
            0,
            1,
            0,
            3,
            1,
            2,
            3,
            1,
            0,
            2,
            3,
            3,
            0,
            3,
            0,
            3,
            0,
            2,
            0,
            0,
            3,
            0,
            2,
            2,
            0,
            0,
            0,
            2,
            3,
            2,
            1,
            0,
            3,
            2,
            1,
            1,
            3,
            1,
            2,
            0,
            3,
            3,
            3,
            3,
            0,
            0,
            0,
            0,
            0,
            3,
            0,
            2,
            0,
            2,
            2,
            0,
            1,
            0,
            2,
            0,
            0,
            2,
            3,
            2,
            2,
            1,
            0,
            0,
            0,
            0,
            2,
            2,
            2,
            0,
            0,
            3,
            2,
            1,
            0,
            0,
            1,
            0,
            1,
            3,
            2,
            1,
            1,
            0,
            0,
            0,
            3,
            1,
            0,
            3,
            1,
            1,
            0,
            0,
            1,
            2,
            0,
            0,
            2,
            1,
            2,
            3,
            2,
            0,
            0,
            2,
            2,
            2,
            0,
            0,
            3,
            2,
            3,
            0,
            1,
            2,
            0,
            2,
            0,
            3,
            3,
            0,
            0,
            0,
            2,
            0,
            3,
            1,
            3,
            0,
            2,
            2,
            1,
            2,
            0,
            2,
            0,
            0,
            0,
            0,
            3,
            1,
            0,
            1,
            0,
            0,
            0,
            1,
            1,
            3,
            3,
            1,
            0,
            0,
            0,
            0,
            3,
            0,
            1,
            2,
            3,
            3,
            0,
            3,
            0,
            2,
            1,
            2,
            1,
            2,
            3,
            2,
            3,
            3,
            0,
            0,
            1,
            3,
            1,
            2,
            2,
            2,
            1,
            3,
            1,
            0,
            0,
            1,
            0,
            1,
            1,
            1,
            2,
            1,
            2,
            1,
            3,
            3,
            0,
            0,
            2,
            0,
            0,
            0,
            3,
            0,
            0,
            2,
            0,
            3,
            0,
            0,
            0,
            0,
            1,
            1,
            1,
            0,
            0,
            0,
            3,
            1,
            1,
            0,
            0,
            3,
            3,
            2,
            1,
            3,
            0,
            0,
            0,
            1,
            0,
            2,
            1,
            2,
            2,
            3,
            1,
            0,
            3,
            0,
            2,
            3,
            3,
            3,
            0,
            2,
            0,
            3,
            1,
            0,
            2,
            0,
            1,
            3,
            1,
            2,
            1,
            2,
            0,
            0,
            1,
            3,
            2,
            2,
            3,
            3,
            0,
            0,
            0,
            0,
            3,
            2,
            0,
            2,
            2,
            0,
            1,
            3,
            0,
            1,
            2,
            3,
            0,
            3,
            2,
            0,
            0,
            0,
            3,
            1,
            0,
            2,
            3,
            0,
            0,
            0,
            0,
            2,
            0,
            0,
            0,
            0,
            0,
            3,
            1,
            0,
            3,
            1,
            3,
            2,
            1,
            1,
            2,
            2,
            2,
            1,
            3,
            3,
            2,
            2,
            3,
            1,
            0,
            3,
            0,
            1,
            0,
            3,
            3,
            3,
            2,
            0,
            3,
            3,
            3,
            2,
            0,
            0,
            0,
            2,
            0,
            3,
            3,
            0,
            0,
            3,
            1,
            2,
            0,
            0,
            0,
            0,
            0,
            0,
            2,
            2,
            1,
            3,
            0,
            0,
            1,
            1,
            0,
            3,
            0,
            0,
            3,
            3,
            0,
            2,
            0,
            2,
            2,
            2,
            1,
            0,
            3,
            3,
            3,
            0,
            2,
            3,
            2,
            1,
            3,
            0,
            0,
            3,
            0,
            0,
            2,
            0,
            0,
            2,
            1,
            0,
            3,
            1,
            0,
            2,
            0,
            1,
            0,
            3,
            1,
            2,
            0,
            0,
            0,
            1,
            1,
            0,
            1,
            0,
            0,
            0,
            2,
            0,
            1,
            0,
            0,
            2,
            0,
            0,
            0,
            0,
            2,
            1,
            0,
            2,
            2,
            1,
            0,
            3,
            1,
            2,
            1,
            2,
            1,
            3,
            3,
            0,
            2,
            0,
            3,
            2,
            0,
            1,
            1,
            3,
            0,
            3,
            2,
            0,
            2,
            0,
            1,
            1,
            0,
            0,
            2,
            0,
            3,
            0,
            0,
            2,
            1,
            3,
            0,
            1,
            0,
            0,
            0,
            0,
            0,
            1,
            2,
            0,
            0,
            0,
            0,
            2,
            0,
            1,
            0,
            3,
            1,
            0,
            1,
            2,
            1,
            2,
            3,
            1,
            0,
            1,
            0,
            0,
            3,
            3,
            2,
            2,
            0,
            1,
            3,
            2,
            0,
            2,
            2,
            2,
            3,
            3,
            0,
            1,
            1,
            1,
            2,
            2,
            3,
            0,
            2,
            0,
            3,
            2,
            1,
            0,
            2,
            0,
            0,
            3,
            0,
            2,
            1,
            2,
            0,
            2,
            1,
            3,
            2,
            3,
            2,
            1,
            0,
            1,
            0,
            3,
            0,
            0,
            0,
            0,
            3,
            1,
            1,
            0,
            2,
            3,
            2,
            3,
            3,
            0,
            0,
            0,
            1,
            2,
            0,
            2,
            3,
            3,
            0,
            3,
            2,
            0,
            1,
            0,
            0,
            0,
            1,
            0,
            1,
            2,
            0,
            3,
            2,
            3,
            3,
            0,
            0,
            0,
            1,
            0,
            0,
            2,
            0,
            2,
            2,
            1,
            3,
            1,
            2,
            0,
            0,
            2,
            2,
            3,
            2,
            0,
            2,
            3,
            3,
            0,
            0,
            1,
            0,
            3,
            0,
            0,
            0,
            0,
            0,
            1,
            2,
            0,
            0,
            0,
            2,
            3,
            0,
            0,
            2,
            1,
            0,
            0,
            3,
            2,
            0,
            0,
            1,
            3,
            3,
            0,
            0,
            1,
            3,
            0,
            0,
            2,
            3,
            0,
            3,
            0,
            1,
            1,
            0,
            3,
            2,
            3,
            0,
            2,
            2,
            0,
            2,
            1,
            3,
            2,
            3,
            3,
            0,
            3,
            0,
            0,
            0,
            0,
            0,
            0,
            2,
            1,
            1,
            0,
            2,
            1,
            0,
            2,
            3,
            0,
            0,
            3,
            3,
            1,
            2,
            1,
            2,
            0,
            2,
            0,
            0,
            1,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            1,
            1,
            0,
            2,
            3,
            3,
            2,
            1,
            1,
            3,
            3,
            3,
            0,
            2,
            2,
            3,
            1,
            0,
            0,
            3,
            0,
            0,
            3,
            2,
            2,
            1,
            2,
            2,
            1,
            1,
            3,
            0,
            0,
            0,
            0,
            0,
            0,
            2,
            1,
            2,
            0,
            2,
            0,
            0,
            0,
            0,
            0,
            0,
            2,
            0,
            2,
            0,
            0,
            3,
            1,
            1,
            0,
            0,
            1,
            0,
            3,
            1,
            3,
            0,
            0,
            1,
            2,
            2,
            3,
            3,
            2,
            3,
            3,
            2,
            0,
            0,
            1,
            1,
            0,
            2,
            1,
            1,
            3,
            3,
            2,
            3,
            0,
            1,
            3,
            1,
            0,
            0,
            3,
            1,
            2,
            3,
            1,
            3,
            2,
            1,
            0,
            0,
            1,
            0,
            2,
            3,
            2,
            0,
            0,
            2,
            1,
            2,
            3,
            3,
            1,
            1,
            0,
            2,
            0,
            3,
            1,
            3,
            0,
            0,
            3,
            1,
            0,
            0,
            0,
            0,
            3,
            0,
            0,
            3,
            3,
            0,
            0,
            0,
            2,
            0,
            1,
            2,
            0,
            0,
            2,
            0,
            3,
            3,
            2,
            1,
            0,
            0,
            0,
            3,
            0,
            3,
            3,
            3,
            1,
            1,
            0,
            0,
            0,
            0,
            0,
            1,
            3,
            3,
            0,
            1,
            2,
            3,
            1,
            0,
            2,
            3,
            0,
            0,
            3,
            0,
            0,
            0,
            1,
            2,
            0,
            1,
            0,
            1,
            3,
            1,
            1,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            1,
            1,
            1,
            0,
            0,
            2,
            3,
            0,
            2,
            1,
            2,
            2,
            3,
            1,
            2,
            0,
            0,
            3,
            0,
            1,
            3,
            0,
            2,
            3,
            3,
            0,
            3,
            0,
            2,
            1,
            0,
            0,
            2,
            1,
            0,
            0,
            0,
            0,
            0,
            0,
            1,
            1,
            0,
            0,
            2,
            0,
            2,
            1,
            3,
            0,
            1,
            3,
            2,
            1,
            0,
            0,
            1,
            2,
            2,
            2,
            2,
            2,
            2,
            0,
            3,
            1,
            1,
            1,
            1,
            0,
            2,
            1,
            2,
            0,
            1,
            0,
            1,
            0,
            3,
            0,
            2,
            3,
            1,
            0,
            0,
            0,
            2,
            2,
            2,
            3,
            0,
            0,
            0,
            0,
            3,
            2,
            0,
            1,
            0,
            1,
            2,
            1,
            1,
            0,
            0,
            2,
            0,
            0,
            1,
            3,
            1,
            1,
            0,
            3,
            1,
            3,
            2,
            0,
            2,
            0,
            1,
            0,
            1,
            1,
            1,
            3,
            0,
            1,
            3,
            2,
            0,
            1,
            1,
            0,
            1,
            1,
            3,
            0,
            1,
            3,
            3,
            2,
            1,
            0,
            0,
            0,
            0,
            3,
            0,
            0,
            2,
            0,
            0,
            0,
            1,
            0,
            0,
            0,
            2,
            1,
            2,
            2,
            2,
            0,
            2,
            0,
            0,
            2,
            2,
            3,
            2,
            3,
            1,
            0,
            0,
            1,
            3,
            3,
            1,
            1,
            3,
            3,
            0,
            0,
            3,
            2,
            0,
            1,
            0,
            3,
            2,
            0,
            2,
            0,
            3,
            3,
            1,
            1,
            3,
            0,
            3,
            0,
            2,
            1,
            3,
            0,
            3,
            0,
            1,
            3,
            1,
            1,
            1,
            2,
            2,
            1,
            2,
            2,
            2,
            1,
            0,
            1,
            2,
            3,
            1,
            1,
            0,
            3,
            0,
            3,
            2,
            3,
            0,
            0,
            2,
            0,
            2,
            0,
            3,
            0,
            2,
            0,
            2,
            0,
            0,
            3,
            1,
            1,
            3,
            3,
            1,
            1,
            2,
            0,
            2,
            1,
            2,
            1,
            3,
            0,
            0,
            2,
            3,
            3,
            0,
            2,
            0,
            1,
            0,
            3,
            2,
            2,
            1,
            0,
            1,
            0,
            2,
            0,
            0,
            2,
            0,
            0,
            1,
            0,
            3,
            0,
            2,
            3,
            3,
            3,
            2,
            0,
            1,
            2,
            3,
            2,
            1,
            2,
            0,
            1,
            2,
            0,
            0,
            0,
            0,
            3,
            1,
            2,
            2,
            1,
            3,
            0,
            0,
            0,
            3,
            1,
            0,
            3,
            1,
            2,
            0,
            0,
            3,
            0,
            2,
            3,
            0,
            3,
            3,
            0,
            3,
            1,
            1,
            1,
            0,
            1,
            0,
            3,
            0,
            0,
            3,
            3,
            3,
            3,
            2,
            0,
            0,
            2,
            1,
            0,
            2,
            0,
            0,
            3,
            0,
            2,
            2,
            3,
            0,
            1,
            0,
            1,
            0,
            2,
            3,
            2,
            3,
            0,
            0,
            0,
            3,
            2,
            2,
            0,
            0,
            0,
            2,
            0,
            2,
            1,
            3,
            1,
            3,
            3,
            1,
            0,
            2,
            2,
            0,
            3,
            2,
            0,
            0,
            1,
            3,
            0,
            0,
            0,
            3,
            1,
            3,
            0,
            2,
            0,
            0,
            3,
            2,
            2,
            3,
            3,
            2,
            3,
            1,
            1,
            2,
            0,
            0,
            1,
            1,
            2,
            0,
            0,
            3,
            3,
            0,
            1,
            0,
            3,
            0,
            2,
            2,
            2,
            2,
            1,
            3,
            0,
            1,
            0,
            2,
            0,
            3,
            2,
            3,
            0,
            0,
            1,
            2,
            0,
            2,
            0,
            0,
            1,
            0,
            0,
            1,
            0,
            1,
            0,
            2,
            3,
            1,
            1,
            3,
            3,
            3,
            0,
            0,
            2,
            0,
            1,
            0,
            1,
            1,
            3,
            2,
            0,
            1,
            3,
            0,
            3,
            1,
            3,
            0,
            0,
            1,
            2,
            1,
            2,
            3,
            2,
            0,
            2,
            1,
            0,
            0,
            0,
            0,
            2,
            2,
            2,
            3,
            0,
            2,
            0,
            2,
            0,
            1,
            2,
            2,
            0,
            0,
            2,
            0,
            1,
            2,
            0,
            1,
            0,
            3,
            3,
            1,
            2,
            1,
            1,
            1,
            3,
            0,
            1,
            0,
            1,
            0,
            3,
            0,
            2,
            0,
            2,
            2,
            1,
            0,
            0,
            1,
            2,
            1,
            2,
            0,
            0,
            3,
            1,
            3,
            2,
            1,
            0,
            3,
            0,
            3,
            0,
            0,
            2,
            0,
            1,
            0,
            0,
            0,
            0,
            3,
            0,
            2,
            3,
            2,
            3,
            0,
            0,
            2,
            1,
            2,
            3,
            2,
            0,
            2,
            3,
            2,
            2,
            0,
            0,
            2,
            2,
            0,
            0,
            3,
            2,
            1,
            0,
            0,
            0,
            0,
            2,
            0,
            3,
            1,
            0,
            3,
            0,
            0,
            0,
            2,
            0,
            2,
            0,
            0,
            1,
            2,
            2,
            0,
            1,
            1,
            2,
            1,
            3,
            0,
            3,
            0,
            3,
            2,
            0,
            0,
            0,
            2,
            3,
            2,
            0,
            1,
            1,
            0,
            0,
            0,
            3,
            0,
            3,
            0,
            1,
            3,
            0,
            3,
            0,
            2,
            0,
            0,
            0,
            2,
            2,
            0,
            0,
            1,
            0,
            3,
            0,
            2,
            3,
            0,
            0,
            3,
            2,
            3,
            3,
            0,
            2,
            3,
            0,
            3,
            0,
            3,
            2,
            0,
            1,
            0,
            3,
            0,
            1,
            0,
            0,
            0,
            1,
            0,
            2,
            0,
            3,
            2,
            3,
            0,
            1,
            0,
            0,
            1,
            0,
            0,
            2,
            0,
            0,
            1,
            3,
            0,
            1,
            0,
            0,
            3,
            2,
            0,
            0,
            0,
            2,
            1,
            1,
            0,
            0,
            0,
            0,
            0,
            2,
            0,
            0,
            1,
            0,
            1,
            0,
            0,
            0,
            0,
            3,
            0,
            0,
            2,
            0,
            0,
            3,
            1,
            3,
            2,
            2,
            3,
            0,
            1,
            3,
            3,
            2,
            0,
            1,
            3,
            0,
            3,
            3,
            3,
            0,
            0,
            0,
            2,
            0,
            1,
            1,
            0,
            2,
            2,
            0,
            0,
            0,
            2,
            3,
            3,
            2,
            3,
            0,
            2,
            1,
            1,
            3,
            0,
            3,
            0,
            0,
            1,
            0,
            2,
            1,
            2,
            2,
            2,
            1,
            0,
            2,
            0,
            1,
            0,
            0,
            0,
            0,
            3,
            0,
            1,
            1,
            2,
            1,
            0,
            0,
            3,
            3,
            2,
            1,
            0,
            0,
            0,
            3,
            1,
            2,
            3,
            1,
            1,
            3,
            2,
            0,
            0,
            3,
            0,
            1,
            0,
            0,
            0,
            1,
            0,
            0,
            1,
            1,
            2,
            0,
            0,
            2,
            3,
            2,
            0,
            1,
            0,
            0,
            0,
            0,
            3,
            3,
            0,
            0,
            1,
            2,
            3,
            2,
            1,
            2,
            0,
            3,
            0,
            0,
            2,
            2,
            3,
            0,
            0,
            0,
            0,
            3,
            0,
            0,
            0,
            0,
        ]

    project_orchestrator.add_plot(
        "Weather", weather, "exterior_air_temperature"
    )
    project_orchestrator.add_plot(
        "Tint", thermal_space_simplified, "inside_air_temperatures"
    )
    project_orchestrator.add_plot("Qwall", simplified_wall_losses, "q_walls")
    project_orchestrator.add_plot(
        "Qprovided", infinite_power_generator, "q_provided"
    )
    # Run project_orchestrator
    project_orchestrator.run()

    src_file_path = resource_filename("tests", "data")
    json_file_path = os.path.join(src_file_path, "house_minimal.json")

    # uncomment to reset template
    # with open(json_file_path, "w") as json_file:
    #     json_file.write(json.dumps(template, indent=4))

    with open(json_file_path, "r") as json_file:
        json_str = json_file.read()
        template_template = json.loads(json_str)

    assert template == template_template

    # Plots
    # project_orchestrator.plot()


def merge_dicts_recursive(dict1, dict2):
    result = dict1.copy()  # Start with a copy of dict1
    for key, value in dict2.items():
        if key in result:
            if isinstance(result[key], dict) and isinstance(value, dict):
                # Recursively merge nested dictionaries
                result[key] = merge_dicts_recursive(result[key], value)
            elif isinstance(result[key], list) and isinstance(value, list):
                # Append lists, if id not present yet
                has_object = False
                for item in result[key]:
                    for o in value:
                        if "id" in item and "id" in o and item["id"] == o["id"]:
                            has_object = True
                if not has_object:
                    result[key].extend(value)

            else:
                # Replace non-list, non-dict values with value from dict2
                result[key] = value
        else:
            # If the key is not in dict1, add it
            result[key] = value
    return result


if __name__ == "__main__":
    test_field_mixin()
    test_generate_simple_house()
